@@ -24,10 +24,12 @@ use Carp;
 
 use Data::Dumper;
 
+use constant MAIN_TAG	=> 'h5';
+
 use vars qw($VERSION %FIELDS $AUTOLOAD %STATUS_DESCR);
 
 BEGIN {
-	$VERSION = '0.27';
+	$VERSION = '0.33';
 
 	%STATUS_DESCR = (
 		0 => 'Empty',
@@ -354,7 +356,7 @@ sub _content {
 		my $crit = shift || '';
 		my $page;
 	
-		$self->code($crit) if $crit =~ /^\d+$/;
+		$self->code($crit) if $crit =~ /^\d{7}$/;
 		$page = $self->_cacheObj()->get($crit) if $self->_cache();
 		
 		$self->_show_message("CRIT: $crit", 'DEBUG');
@@ -363,6 +365,7 @@ sub _content {
 			if( -f $crit ) {
 				$self->_show_message("Parse IMDB HTML file ...", 'DEBUG');
 				
+				local $/;
 				undef $/;
 				open FILE, $crit or die "Cannot open off-line IMDB file: $!!";
 				$page = <FILE>;
@@ -386,8 +389,8 @@ sub _content {
 		
 		$self->{content} = \$page;
 	}
-	
-	return $self->{content};
+
+	$self->{content};
 }
 
 sub _get_page_from_internet {
@@ -428,6 +431,35 @@ sub _parser {
 	return $self->{parser};
 }
 
+=item _get_simple_prop()
+
+Retrieve a simple movie property which surrownded by <B>.
+
+=cut
+sub _get_simple_prop {
+	my CLASS_NAME $self = shift;
+	my $target = shift || '';
+	
+	my $parser = $self->_parser(FORCED);
+
+	while(my $tag = $parser->get_tag(MAIN_TAG)) {
+		my $text = $parser->get_text;
+
+		$self->_show_message("[$tag->[0]] $text --> $target", 'DEBUG');
+
+		last if $text =~ /$target/i;
+	}
+
+	my $end_tag = $target eq 'trivia' ? '/div' : 'a';
+
+	my $res = $parser->get_trimmed_text($end_tag);	
+
+	$self->_show_message($res, 'DEBUG');
+	
+	return $res;
+}
+
+
 =back
 
 =cut
@@ -436,22 +468,41 @@ sub _search_results {
 	my CLASS_NAME $self = shift;
 	my $pattern = shift || croak 'Please, specify search pattern!';
 	my $end_tag = shift || '/li';
+	my $year	= shift;
 	
-	my @matched;
+	my(@matched, @guess_res, %matched_hash);
 	my $parser = $self->_parser();
-
+	
+	my $count = 0;
 	while( my $tag = $parser->get_tag('a') ) {
 		my $href = $tag->[1]{href};
-		if( my($id) = $href =~ /$pattern/ ) {
-			push @matched, {id => $id, title => $parser->get_trimmed_text('a', $end_tag)};
+		my $title = $parser->get_trimmed_text('a', $end_tag);
+		
+		next if $title =~ /\[IMG\]/i;
+
+		if(my($id) = $href =~ /$pattern/) {
+			$matched_hash{$id} = {title => $title, 'pos' => $count++};
+			@guess_res = ($id, $title) if $year && $title =~ /$year/ && !@guess_res;
 		}	
 	}
 
+	@matched = map { {title => $matched_hash{$_}->{title}, id => $_} } 
+				sort { $matched_hash{$a}->{'pos'} <=> $matched_hash{$b}->{'pos'} } keys %matched_hash;
+	
 	$self->matched(\@matched);
-	$self->_content($matched[0]->{id});
+	
+	my($title, $id);
+	if(@guess_res) {
+		($id, $title) = @guess_res;
+	} else {
+		$title = $matched[0]->{title};
+		$id = $matched[0]->{id};
+	}
+
+	$self->_content($id);
 	$self->_parser(FORCED);
 
-	return $matched[0]->{title};
+	return $title;
 }
 
 =item matched()
@@ -541,11 +592,11 @@ HTML::TokeParser
 
 =head1 AUTHOR
 
-Mikhail Stepanov (stepanov.michael@gmail.com)
+Mikhail Stepanov AKA nite_man (stepanov.michael@gmail.com)
 
 =head1 COPYRIGHT
 
-Copyright (c) 2004 - 2005, Mikhail Stepanov. All Rights Reserved.
+Copyright (c) 2004 - 2007, Mikhail Stepanov.
 This module is free software. It may be used, redistributed and/or 
 modified under the same terms as Perl itself.
 
