@@ -53,7 +53,10 @@ use Carp;
 use Data::Dumper;
 
 use fields qw(	_title
+		_kind
 				_year
+				_episodes
+				_episodeof
 				_summary
 				_cast
 				_directors
@@ -74,10 +77,11 @@ use fields qw(	_title
 				_awards
 				_official_sites
 				_release_dates
+				_aspect_ratio
 				full_plot_url
 		);
 	
-use vars qw( $VERSION %FIELDS %FILM_CERT $PLOT_URL );
+use vars qw( $VERSION %FIELDS %FILM_CERT %FILM_KIND $PLOT_URL );
 
 use constant CLASS_NAME 	=> 'IMDB::Film';
 use constant FORCED			=> 1;
@@ -87,7 +91,7 @@ use constant EMPTY_OBJECT	=> 0;
 use constant MAIN_TAG		=> 'h5';
 
 BEGIN {
-		$VERSION = '0.27';
+		$VERSION = '0.33';
 						
 		# Convert age gradation to the digits		
 		# TODO: Store this info into constant file
@@ -95,7 +99,17 @@ BEGIN {
 						R 		=> 16, 
 						'NC-17' => 16, 
 						PG 		=> 13, 
-						'PG-13' => 13 );							
+						'PG-13' => 13
+					);							
+
+		%FILM_KIND = ( 	''		=> 'movie',
+						TV		=> 'tv movie',
+						V		=> 'video movie',
+						mini	=> 'tv mini series',
+						VG		=> 'video game',
+						S		=> 'tv series',
+						E		=> 'episode'
+					);				
 }
 
 {
@@ -108,7 +122,7 @@ BEGIN {
 		matched			=> [],
         host			=> 'www.imdb.com',
         query			=> 'title/tt',
-        search 			=> 'find?tt=on;mx=20;q=',		
+        search 			=> 'find?s=tt;q=',	
 		status			=> 0,		
 		timeout			=> 10,
 		user_agent		=> 'Mozilla/5.0',
@@ -116,6 +130,7 @@ BEGIN {
 		_also_known_as	=> [],
 		_official_sites	=> [],
 		_release_dates	=> [],
+		_duration		=> [],
 	);	
 	
 	sub _get_default_attrs { keys %_defaults }		
@@ -157,7 +172,7 @@ sub _init {
 	
 	$self->SUPER::_init(%args);
 	
-	$self->title(FORCED);
+	$self->title(FORCED, \%args);
 	
 	unless($self->title) {
 		$self->status(EMPTY_OBJECT);
@@ -177,6 +192,13 @@ sub _init {
 
 =over 4
 
+=item year
+
+Define a movie's year. It's useful to use it to get the proper movie by its title:
+
+	my $imdbObj = new IMDB::Film(crit => 'Jack', year => 2003);
+	print "Got #" . $imdbObj->code . " " . $imdbObj->title . "\n"; #0379836
+
 =item proxy
 
 defines proxy server name and port:
@@ -270,32 +292,11 @@ Implemets functionality to search film by name.
 =cut
 sub _search_film {
 	my CLASS_NAME $self = shift;
+	my $args = shift || {};
 
-	return $self->SUPER::_search_results('\/title\/tt(\d+)');
+	return $self->SUPER::_search_results('\/title\/tt(\d+)', '/td', $args->{year});
 }
 
-=item _get_simple_prop()
-
-Retrieve a simple movie property which surrownded by <B>.
-
-=cut
-sub _get_simple_prop {
-	my CLASS_NAME $self = shift;
-	my $target = shift || '';
-	
-	my $parser = $self->_parser(FORCED);
-
-	while(my $tag = $parser->get_tag(MAIN_TAG)) {
-		my $text = $parser->get_text;
-		last if $text =~ /$target/i;
-	}
-
-	my $res = $parser->get_trimmed_text(MAIN_TAG, 'a');
-	
-	return $res;
-}
-
-
 =back
 
 =head2 Object Public Methods
@@ -333,7 +334,9 @@ matched films and continue to process first one:
 =cut
 sub title {	
 	my CLASS_NAME $self = shift;
-	my $forced = shift || 0;
+	my $forced 	= shift || 0;
+	my $args	= shift || {};
+
 	if($forced) {
 		my $parser = $self->_parser(FORCED);
 	
@@ -341,21 +344,44 @@ sub title {
 		my $title = $parser->get_text();
 		if($title =~ /imdb\s+title\s+search/i) {
 			$self->_show_message("Go to search page ...", 'DEBUG');
-			$title = $self->_search_film();				
+			$title = $self->_search_film($args);				
 		} 
 		
 		if($title) {
 			$self->retrieve_code($parser, '/pro.imdb.com/title/tt(\d+)') 
 														unless $self->code;
 			$title =~ s/\*/\\*/g;
-
-			($self->{_title}, $self->{_year}) = $title =~ m!(.*?)\s+\((\d{4}).*?\)!;
+			
+			($self->{_title}, $self->{_year}, $self->{_kind}) = $title =~ m!(.*?)\s+\(([\d\?]{4}).*?\)(?:\s+\((.*?)\))?!;
+			$self->{_kind} = '' unless $self->{_kind};
+			
+        		#"The Series" An Episode (2005)
+			#"The Series" (2005)
+        		if( $self->{_title} =~ /\"[^\"]+\"(\s+.+\s+)?/ ) {
+        			$self->{_kind} = $1 ? 'E' : 'S';
+        		}
+        		
+			
 		}	
 	}	
 	
 	return $self->{_title};
 }
 
+=item kind()
+
+Get kind of movie:
+	
+	my $kind = $film->kind();
+	
+	Possible values are: 'movie', 'tv series', 'tv mini series', 'video game', 'video movie', 'tv movie', 'episode'.
+
+=cut
+sub kind {
+	my CLASS_NAME $self = shift;
+	return $FILM_KIND{$self->{_kind}};
+}
+
 =item year()
 
 Get film year:
@@ -368,6 +394,102 @@ sub year {
 	return $self->{_year};
 }
 
+
+=item episodes()
+
+Retrieve episodes info list each element of which is hash reference for tv series -
+{ id => <ID>, title => <Title>, season => <Season>, episode => <Episode>, date => <Date>, plot => <Plot> }:
+
+	my @episodes = @{ $film->episodes() };
+
+=cut
+sub episodes {
+	my CLASS_NAME $self = shift;
+
+	return if $self->kind ne "tv series";
+
+	unless($self->{_episodes}) {
+		my $page;
+		$page = $self->_cacheObj->get($self->code . '_episodes') if $self->_cache;
+
+		unless($page) {
+			my $url = "http://". $self->{host} . "/" . $self->{query} .  $self->code . "/episodes";
+			$self->_show_message("URL for episodes is $url ...", 'DEBUG');
+
+			$page = $self->_get_page_from_internet($url);
+			$self->_cacheObj->set($self->code.'_episodes', $page, $self->_cache_exp) if $self->_cache;
+		}
+
+		my $parser = $self->_parser(FORCED, \$page);
+		while(my $tag = $parser->get_tag('a')) {
+			next unless $tag->[1]->{name} and $tag->[1]->{name} =~ /year/;
+			$parser->get_tag('h4');
+			my($season, $episode) = $parser->get_text =~ /Season\s+(.*?),\s+Episode\s+([^:]+)/;
+			my $imdb_tag = $parser->get_tag('a');
+			my($id) = $imdb_tag->[1]->{href} =~ /(\d+)/;
+			my $title = $parser->get_trimmed_text;
+			$parser->get_tag('b');
+			my($date) = $parser->get_trimmed_text =~ /Original Air Date:\s+(.*)/;
+			$parser->get_tag('br');
+			my $plot = $parser->get_trimmed_text;
+			
+			push @{ $self->{_episodes} }, { 	
+								season 	=> $season, 
+								episode => $episode, 
+								id 		=> $id,
+								title 	=> $title,
+								date 	=> $date,
+								plot 	=> $plot
+							};
+		}
+	}
+
+	return $self->{_episodes};
+}
+
+=item episodeof()
+
+Retrieve parent tv series list each element of which is hash reference for episode -
+{ id => <ID>, title => <Title>, year => <Year> }:
+
+	my @tvseries = @{ $film->episodeof() };
+
+=cut
+sub episodeof {
+   my CLASS_NAME $self = shift;
+   my $forced = shift || 0;
+
+   return if $self->kind ne "episode";
+
+   if($forced) {
+	   my($episodeof, $title, $year, $episode, $season, $id);
+	   my($parser) = $self->_parser(FORCED);
+
+	   while($parser->get_tag(MAIN_TAG)) {
+		   last if $parser->get_text =~ /^TV Series/i;
+	   }
+
+	   while(my $tag = $parser->get_tag('a')) {
+		   ($title, $year) = ($1, $2) if $parser->get_text =~ m!(.*?)\s+\(([\d\?]{4}).*?\)!;
+		   last unless $tag->[1]{href} =~ /title/i;
+		   ($id) = $tag->[1]{href} =~ /(\d+)/;
+	   }
+
+	   #start again
+	   $parser = $self->_parser(FORCED);
+	   while($parser->get_tag(MAIN_TAG)) {
+		   last if $parser->get_text =~ /^Original Air Date/i;
+	   }
+	   
+	   $parser->get_token;	   
+	   ($season, $episode) = $parser->get_text =~ /\(Season\s+(\d+),\s+Episode\s+(\d+)/;
+
+	   push @{ $self->{_episodeof} }, {title => $title, year => $year, id => $id, season => $season, episode => $episode};
+   }
+
+   return $self->{_episodeof};
+}
+
 =item cover()
 
 Retrieve url of film cover:
@@ -457,7 +579,7 @@ sub writers {
 		my (@writers, $tag);
 		
 		while($tag = $parser->get_tag(MAIN_TAG)) {
-			last if $parser->get_text =~ /writ(?:ing|ers)/i;
+			last if $parser->get_text =~ /writ(?:ing|ers|er)/i;
 		}
 			
 		while($tag = $parser->get_tag()) {
@@ -588,7 +710,7 @@ sub rating {
 		my $tag = $parser->get_tag('b');	
 		my $text = $parser->get_trimmed_text('b', '/a');
 
-		my($rating, $val) = $text =~ m!(\d+\.?\d+)\/.*?\((\d+\,?\d+)\s.*?\)?!;
+		my($rating, $val) = $text =~ m!(\d*\.?\d*)\/.*?\((\d*\,?\d*)\s.*?\)?!;
 		$val =~ s/\,// if $val;
 
 		$self->{_rating} = [$rating, $val];
@@ -617,20 +739,17 @@ sub cast {
 		my (@cast, $tag, $person, $id, $role);
 		my $parser = $self->_parser(FORCED);
 	
-		#while($tag = $parser->get_tag(MAIN_TAG)) {
-		while($tag = $parser->get_tag('img')) {
-			#next unless (exists  $tag->[1]{class} and $tag->[1]{class} eq 'blackcatheader');
-			#last if $parser->get_text =~ /^(cast overview|credited cast|(?:series )?complete credited cast)/i;
-			last if $tag->[1]->{src} =~ /cast/i;
+		while($tag = $parser->get_tag('table')) {
+			last if $tag->[1]->{class} && $tag->[1]->{class} =~ /cast/i;
 		}
 		
 		while($tag = $parser->get_tag('a')) {
 			last if $tag->[1]{href} =~ /fullcredits/i;
-			#if($tag->[1]{href} && $tag->[1]{href} =~ m!/name/nm(\d+?)/!) {
-			if($tag->[1]{href} && $tag->[1]{href} =~ m#(?<!tinyhead)/name/nm(\d+?)/#) {
+			if($tag->[1]{href} && !$tag->[1]{onclick} && $tag->[1]{href} =~ m#name/nm(\d+?)/#) {
 				$person = $parser->get_text;
 				$id = $1;	
-				my $text = $parser->get_trimmed_text('a', '/tr');
+				my $text = $parser->get_trimmed_text('/tr');
+
 				($role) = $text =~ /.*?\s+(.*)$/;			
 				push @cast, {id => $id, name => $person, role => $role};
 			}	
@@ -638,16 +757,20 @@ sub cast {
 		
 		$self->{_cast} = \@cast;
 	}
-
+	
 	return $self->{_cast};
 }
 
 =item duration()
 
-Retrieve film duration in minutes:
+In the scalar context it retrieves a film duration in minutes (the first record):
 
 	my $duration = $film->duration();
 
+In array context it retrieves all movie's durations:
+
+	my @durations = $film->duration();
+
 =cut
 sub duration {
 	my CLASS_NAME $self = shift;
@@ -661,10 +784,12 @@ sub duration {
 			last if $text =~ /runtime:/i;
 		}	
 		
-		$self->{_duration} = $parser->get_trimmed_text(MAIN_TAG, 'br');
+		my @runtime = split /\s+(\/|\|)\s+/, $parser->get_trimmed_text(MAIN_TAG, 'br');
+		
+		$self->{_duration} = \@runtime;		
 	}
 
-	return $self->{_duration};
+	return wantarray ? @{ $self->{_duration} } : $self->{_duration}->[0];
 }
 
 =item country()
@@ -686,9 +811,11 @@ sub country {
 
 		my (@countries);
 		while(my $tag = $parser->get_tag()) {
-			
+
 			if( $tag->[0] eq 'a' && $tag->[1]{href} && $tag->[1]{href} =~ /countries/i ) {
-				push @countries, $parser->get_text();
+				my $text = $parser->get_text();
+				$text =~ s/\n//g;
+				push @countries, $text;
 			} 
 			
 			last if $tag->[0] eq 'br';
@@ -721,7 +848,9 @@ sub language {
 		while($tag = $parser->get_tag()) {
 			
 			if( $tag->[0] eq 'a' && $tag->[1]{href} && $tag->[1]{href} =~ /languages/i ) {
-				push @languages, $parser->get_text();
+				my $text = $parser->get_text();
+				$text =~ s/\n//g;
+				push @languages, $text;
 			} 
 			
 			last if $tag->[0] eq 'br';
@@ -788,7 +917,7 @@ sub goofs {
 	my CLASS_NAME $self = shift;
 
 	$self->{_goofs} = $self->_get_simple_prop('goofs') unless($self->{_goofs});
-	return $self->{_trivia};
+	return $self->{_goofs};
 }
 
 =item awards()
@@ -806,6 +935,21 @@ sub awards {
 	return $self->{_awards};
 }
 
+=item aspect_ratio()
+
+Returns an aspect ratio of specified movie:
+
+	my $aspect_ratio = $film->aspect_ratio();
+
+=cut
+sub aspect_ratio {
+	my CLASS_NAME $self = shift;
+
+	$self->{_aspect_ratio} = $self->_get_simple_prop('aspect ratio') unless $self->{_aspect_ratio};
+
+	return $self->{_aspect_ratio};
+}
+
 =item summary()
 
 Retrieve film user summary:
@@ -855,7 +999,9 @@ sub certifications {
 		while($tag = $parser->get_tag()) {
 			
 			if($tag->[0] eq 'a' && $tag->[1]{href} && $tag->[1]{href} =~ /certificates/i) {
-				my($country, $range) = split /\:/, $parser->get_text;
+				my $text = $parser->get_text();
+				$text =~ s/\n//g;
+				my($country, $range) = split /\:/, $text;
 				$cert_list{$country} = $range;
 			}
 
@@ -946,8 +1092,9 @@ sub official_sites {
 	
 
 		my $parser = $self->_parser(FORCED, \$page);
-		while(my $tag = $parser->get_tag('div')) {
-			last if $tag->[1]->{id} && $tag->[1]->{id} eq 'tn15content';
+		while(my $tag = $parser->get_tag()) {
+			#last if $tag->[1]->{id} && $tag->[1]->{id} eq 'tn15title';
+			last if $tag->[0] eq 'ol';
 		}
 
 		while(my $tag = $parser->get_tag()) {
@@ -990,8 +1137,9 @@ sub release_dates {
 		}
 
 		my $parser = $self->_parser(FORCED, \$page);
-		while(my $tag = $parser->get_tag('div')) {
-			last if $tag->[1]->{id} && $tag->[1]->{id} eq 'tn15content';
+		while(my $tag = $parser->get_tag('th')) {
+			#last if $tag->[1]->{id} && $tag->[1]->{id} eq 'tn15title';
+			last if $tag->[1]->{class} && $tag->[1]->{class} eq 'xxxx';
 		}
 
 		while(my $tag = $parser->get_tag()) {
@@ -1002,6 +1150,7 @@ sub release_dates {
 			my $date = $parser->get_text;
 			my $year_tag = $parser->get_tag('a');
 			my $year = $parser->get_text;
+
 			push @{ $self->{_release_dates} }, {country => $country, date => "$date, $year"};
 		}
 	}
@@ -1079,11 +1228,11 @@ http://videoguide.sf.net
 
 =head1 AUTHOR
 
-Michael Stepanov (stepanov.michael@gmail.com)
+Michael Stepanov AKA nite_man (stepanov.michael@gmail.com)
 
 =head1 COPYRIGHT
 
-Copyright (c) 2004 - 2005, Michael Stepanov. All Rights Reserved.
+Copyright (c) 2004 - 2007, Michael Stepanov.
 This module is free software. It may be used, redistributed and/or 
 modified under the same terms as Perl itself.
 
